//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file GammaRaySpectrometry/src/GRSAnalysisManager.cc
/// \brief Implementation of the GRSAnalysisManager class

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

#include "GRSAnalysisManager.hh"
FILE* theFile; 

///---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSAnalysisManager::GRSAnalysisManager(GRSDetectorConstruction* det)
:fFactoryOn(false), fDetector(det)
{}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSAnalysisManager::~GRSAnalysisManager()
{
 delete fDetector; 
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSAnalysisManager::Book()
{
    // create or get analysis manager
    G4AnalysisManager* analysisManager = G4AnalysisManager::Instance();
    analysisManager->SetDefaultFileType("root");
    analysisManager->SetVerboseLevel(1);
    // Only merge in MT mode to avoid warning when running in Sequential mode
    #ifdef G4MULTITHREADED
     analysisManager->SetNtupleMerging(true);
    #endif
 
   // open an output file (first replace '.' to '-' in output file for ROOT)
   G4String tmpSt = fDetector->GetFileName();
   std::replace( tmpSt.begin(), tmpSt.end(), '.', '-');
 
    //G4bool fileOpen = analysisManager->OpenFile(tmpSt);

   G4bool fileOpen = analysisManager->OpenFile( fDetector->GetFileName() );
    
    if (! fileOpen) {
        G4cerr << "\n---> GRSAnalysisManager::Book(): cannot open "
               << analysisManager->GetFileName() << G4endl;
        return;
    }

    if ( ! fFactoryOn )
    {
    // create 1st ntuple (id = 0)
    analysisManager->CreateNtuple("SRC", "SRC");
    analysisManager->CreateNtupleDColumn(0, "source_energy");
    analysisManager->CreateNtupleDColumn(0, "source_position_X");
    analysisManager->CreateNtupleDColumn(0, "source_position_Y");
    analysisManager->CreateNtupleDColumn(0, "source_position_Z");
    analysisManager->FinishNtuple(0);

    // create 2st ntuple (id = 1)
    analysisManager->CreateNtuple("HITS", "HITS");
    analysisManager->CreateNtupleDColumn(1,"edep_in_germanium");
    analysisManager->FinishNtuple(1);

    fFactoryOn = true;
  }
  G4cout << "\n----> Output file is open in "
         << analysisManager->GetFileName() << "."
         << analysisManager->GetFileType() << G4endl;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSAnalysisManager::Save()
{
   if (! fFactoryOn) return;

    G4AnalysisManager* analysisManager = G4AnalysisManager::Instance();
    analysisManager->Write();
    analysisManager->CloseFile();

    G4cout << "\n ... histograms and ntuples are saved\n" << G4endl;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSAnalysisManager::FillSrcNtuple(G4double sourceEnergy, G4double sourcePosX,G4double sourcePosY,G4double sourcePosZ)
{ 
    G4AnalysisManager* analysisManager = G4AnalysisManager::Instance();
    analysisManager->FillNtupleDColumn(0,0, sourceEnergy);
    analysisManager->FillNtupleDColumn(0,1, sourcePosX);
    analysisManager->FillNtupleDColumn(0,2, sourcePosY);
    analysisManager->FillNtupleDColumn(0,3, sourcePosZ);
    analysisManager->AddNtupleRow(0);
}  

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSAnalysisManager::FillEdepNtuple(G4double edep)
{
    if (edep==0) return;
    G4AnalysisManager* analysisManager = G4AnalysisManager::Instance();
    analysisManager->FillNtupleDColumn(1,0, edep);
    analysisManager->AddNtupleRow(1);
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSAnalysisManager::PrintStatistic(const G4Run* theRun, G4double totalEfficiency, G4double peakEfficiency )
{
    if (!fFactoryOn) return;
    G4cout<<"\n"<<G4endl;
    G4cout << "\n ----> print histograms statistic \n" << G4endl;
    G4cout << "     ---Gamma-Ray-Spectrometry--- RunStatus \n"<<G4endl;
    G4cout <<"       - total events generated  : "<<std::setw(12)<< theRun->GetNumberOfEvent() << G4endl;
    G4cout <<"     [ Peak Efficiency ] "<<G4endl;
    G4cout <<"       - peak count              : "<<std::setw(12)<<peakEfficiency<<"\n"
           <<"       - peak efficiency         : "<<std::setw(12)<< (double) peakEfficiency / (double) theRun->GetNumberOfEvent()<<"\n"
           <<"       - sigma (%)               : "<<std::setw(12)<< (sqrt(peakEfficiency) /peakEfficiency)*100<<G4endl;
    G4cout <<"     [ Total Efficiency ] "<<G4endl;
    G4cout <<"       - total count             : "<<std::setw(12)<<totalEfficiency<<"\n"
           <<"       - total efficiency        : "<<std::setw(12)<< (double)totalEfficiency/(double)theRun->GetNumberOfEvent()<<"\n"
           <<"       - sigma (%)               : "<<std::setw(12)<< (sqrt(totalEfficiency) /totalEfficiency)*100<<G4endl;
    G4cout<<"\n"<<G4endl;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSAnalysisManager::CreateASCIIDataFile(const G4Run* theRun, G4double totalEfficiency, G4double peakEfficiency)
{
    // create ASCII file
    G4cout <<"GRSAnalysisManager() creating and writing ASCII file"<<G4endl;
    G4String asciiFile;
    using namespace std;
    stringstream buffer;
    buffer <<fDetector->GetFileName()<<".dat";
    asciiFile = buffer.str();
    theFile=fopen(asciiFile, "w");

    fprintf(theFile,"##============================================================ \n");
    fprintf(theFile,"##============================================================ \n");
    fprintf(theFile,"## Geant4 - an Object-Oriented Toolkit for Simulation in HEP   \n");
    fprintf(theFile,"## Gamma-Ray-Spectrometry benchmark                            \n");
    fprintf(theFile,"##============================================================ \n");
    fprintf(theFile,"## L1  full energy peak information (FEP)                      \n");
    fprintf(theFile,"## L2  total energy information (TE)                           \n");
    fprintf(theFile,"##------------------------------------------------------------ \n");
    fprintf(theFile,"## C1  primary energy (keV)                                    \n");
    fprintf(theFile,"## C2  events generated                                        \n");
    fprintf(theFile,"## C3  counting                                                \n");
    fprintf(theFile,"## C4  efficiency                                              \n");
    fprintf(theFile,"## C5  std(per cent)                                           \n");
    fprintf(theFile,"##============================================================ \n");

    fprintf(theFile,"%08.3f %09i %09f %8.6f %8.6f \n", fDetector->GetEnergyOfParticle()/keV, theRun->GetNumberOfEvent(), peakEfficiency, peakEfficiency/ theRun->GetNumberOfEvent(), (sqrt(peakEfficiency)/peakEfficiency)*100);
    fprintf(theFile,"%08.3f %09i %09f %8.6f %8.6f \n", fDetector->GetEnergyOfParticle()/keV, theRun->GetNumberOfEvent(), totalEfficiency, totalEfficiency/theRun->GetNumberOfEvent(), (sqrt(totalEfficiency)/totalEfficiency)*100);

    fclose(theFile); 
    G4cout <<"GRSAnalysisManager() ... saved in "<<fDetector->GetFileName()<<".dat"<< G4endl;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//
