//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// \file GammaRaySpectrometry/src/GRSDetectorConstruction.cc
// \brief Implementation of the DetectorConstruction class

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

#include "GRSDetectorConstruction.hh"

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSDetectorConstruction::GRSDetectorConstruction()
{
  verbose=1;

  startAng = 0*deg; 
  spaceAng = 360*deg;
  forceSolid=0;
  overlapChecking=1;

  addShielding=1;
  addGeDeadZone=1;
  cristalDZLayer=1.0*mm;
  
  sourceType="Water";
  source_diameter=0.0;
  source_height=0.0;
  
  sourcePosition.set(0.0,0.0,1.0);
  sourceTranslation.set(0.0,0.0,0);
  
  particleName="gamma";
  particleEnergy=60.0*keV;
  ionA=1;
  ionZ=1;

  energySigma=1.0*keV;
  detecEThreshold=0.1*eV;

  addOutputFile=true;
  
  fileName="GammaRaySpec";
  
  DefineMaterials();

  detectorMessenger = new GRSDetectorMessenger(this);
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSDetectorConstruction::~GRSDetectorConstruction()
{}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSDetectorConstruction::DefineMaterials()
{
  // Materials

  G4double a;
  G4double z;
  G4double density;
  G4int ncomponents;
  G4double fractionmass;
  G4double temperature;
  G4double pressure;

  // create materials 
  
  Vacuum_mat = new G4Material("GRSVacuum", z=1., a=1.01*g/mole,density= universe_mean_density, kStateGas, temperature=2.73*kelvin, pressure=3.e-18*pascal);
  
  G4Element* C  = new G4Element("C", "C",  z=6.,  a=12.01*g/mole);  // Carbon
  G4Element* N  = new G4Element("N", "N",  z=7.,  a=14.01*g/mole);  // Nitrogen
  G4Element* O  = new G4Element("O", "O",  z=8.,  a=16.00*g/mole);  // Oxygen
  G4Element* Ar = new G4Element("Ar","Ar", z=18., a=39.95*g/mole);  // Argon
  
  Air_mat = new G4Material("GRS-Air", density=1.2048*mg/cm3, ncomponents=4, kStateGas);
  Air_mat->AddElement(C,fractionmass=0.0124*perCent);
  Air_mat->AddElement(N,fractionmass=75.5*perCent);
  Air_mat->AddElement(O,fractionmass=23.2*perCent);
  Air_mat->AddElement(Ar,fractionmass=1.28*perCent);
  
  G4Element* Pb = new G4Element("Pb","Pb", z=82., a=207.200*g/mole); // Lead
  Lead_mat = new G4Material("GRS-Lead", density=11.35*g/cm3, ncomponents=1);
  Lead_mat->AddElement(Pb,1);
  
  G4Element* Ge = new G4Element("Ge","Ge", z=32., a=72.59*g/mole); // Germanium
  Germanium_mat = new G4Material("GRS-Germanium", density= 5.323*g/cm3, ncomponents=1, kStateSolid);
  Germanium_mat->AddElement(Ge,1);
  Germanium_mat->GetIonisation()->SetMeanExcitationEnergy(2.96*eV);
  
  G4Element* Al = new G4Element("Al","Al", z=13., a=26.98*g/mole);  // Aluminum
  Aluminum_mat = new G4Material("GRS-Aluminum", density=2.7*g/cm3, ncomponents=1, kStateSolid);
  Aluminum_mat->AddElement(Al,1);
  
  G4Element* H = new G4Element("H", "H",  z=1.,  a=1.01*g/mole);  // Hydrogen
  Water_mat = new G4Material("GRS-Water",  density=1.0*g/cm3, ncomponents=2, kStateLiquid);
  Water_mat->AddElement(H,2);
  Water_mat->AddElement(O,1);
  
  G4Element* Si = new G4Element("Si","Si", z=14., a=28.085*g/mole); // Silicon
  Dirt_mat = new G4Material("GRS-Dirt",  density=1.4*g/cm3, ncomponents=2);
  Dirt_mat->AddElement(Si,1);
  Dirt_mat->AddElement(O,2);
  
  Cellulose_mat = new G4Material("GRS-Cellulose",  density=0.3*g/cm3, ncomponents=3);
  Cellulose_mat->AddElement(C,6);
  Cellulose_mat->AddElement(H,11);
  Cellulose_mat->AddElement(O,5);
}
//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

G4VPhysicalVolume* GRSDetectorConstruction::Construct()
{
    return ConstructDetector();
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

G4VPhysicalVolume* GRSDetectorConstruction::ConstructDetector()
{
  // detector Construction

  G4cout<<" ------------------------------------------------------------------ "<< G4endl;
  G4cout<<" --- Geant4 - an Object-Oriented Toolkit for Simulation in HEP  --- "<< G4endl;
  G4cout<<" ---                                                            --- "<< G4endl;
  G4cout<<" --- G4/GammaRaySpec                                            --- "<< G4endl;
  G4cout<<" ---                                                            --- "<< G4endl;
  G4cout<<" --- ICRM GRS WG : Monte Carlo action (LNHB)                    --- "<< G4endl;
  G4cout<<" ---                                                            --- "<< G4endl;
  G4cout<<" --- Description: use of Monte Carlo simulation                 --- "<< G4endl;
  G4cout<<" ---                      in Gamma spectrometry                 --- "<< G4endl;
  G4cout<<" ---                                                            --- "<< G4endl;
  G4cout<<" --- Released: September-2020                                   --- "<< G4endl;
  G4cout<<" ---                                                            --- "<< G4endl;
  G4cout<<" ------------------------------------------------------------------ "<< G4endl;
  
  // cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  if(verbose>=1) // print list of materials
  {
      G4cout << G4endl << "The materials defined are : "<<G4endl;
      G4cout << *(G4Material::GetMaterialTable()) <<G4endl;
  }
  
  if(verbose>-1) overlapChecking=true;  // check overlapping in geometry
   
  if(verbose==2){ startAng = 45*deg, spaceAng = 180*deg;} // cut geometry only for visualization test

  // create World volume (experimental space)

  const G4double worldSize = 1.0*m; // default size of experimental space is 1.0 x 1.0 x 1.0 m box

  G4Box* const solidWorld = new G4Box("World",                                        // solid volume name
                                       worldSize/2,                                   // x size
                                       worldSize/2,                                   // y size
                                       worldSize/2);                                  // z size
  G4LogicalVolume* const logicalWorld = new G4LogicalVolume(solidWorld,               // logical volume
                                                            Air_mat,                  // material
                                                            solidWorld->GetName());   // logical volume name
  //logicalWorld->SetVisAttributes(G4VisAttributes::Invisible);                       // volume is always invisible
  //G4RotationMatrix* rota = new G4RotationMatrix();
  //rota->rotateX(90.*deg);
  G4VPhysicalVolume* const physicalWorld = new G4PVPlacement(NULL,                    // no rotation
                                                             G4ThreeVector(0,0,0),    // must be at origin
                                                             logicalWorld,            // the logical volume
                                                             logicalWorld->GetName(), // physical volume name
                                                             NULL,                    // no mother volume
                                                             false,                   // no boolean ops
                                                             0,                       // copy number
                                                             overlapChecking);        // check for overlapping

  if (addShielding==true){

  // create lead shielding 

  G4double shieldSize = 400.0*mm;
  G4double shieldLayer = 50.0*mm;

  G4Tubs* shielding_1 = new G4Tubs("HPGe_SHIELDING",                                  // solid volume name
                                   0.0,                                               // cylinder inner radius
                                   shieldSize/2,                                      // cylinder outer radius
                                   shieldSize/2,                                      // cylinder half Z
                                   startAng,                                          // cylinder start angle
                                   spaceAng);                                         // cylinder spanning angle
 shielding_1_log = new G4LogicalVolume(shielding_1,                                   // logical volume
                                          Lead_mat,                                   // material
                           shielding_1->GetName());                                   // name of logical volume (same that solid volume)
  new G4PVPlacement(NULL,                                                             // no rotation
                    G4ThreeVector(0,0,0),                                             // position (the top of germanium Cristal is at zero according to symmetry axis)
                    shielding_1_log,                                                  // the logical volume
                    shielding_1->GetName(),                                           // name of physical volume (same that solid volume)
                    logicalWorld,                                                     // the mother volume
                    false,                                                            // no boolean ops
                    0,                                                                // copy number
                    overlapChecking);                                                 // check for overlapping
  G4Tubs* shielding_2 = new G4Tubs("HPGe_SHIELDING",                                  // solid volume name
                                   0.0,                                               // cylinder inner radius
                                   shieldSize/2 -shieldLayer,                         // cylinder outer radius
                                   shieldSize/2 -shieldLayer,                         // cylinder half Z
                                   startAng,                                          // cylinder start angle
                                   spaceAng);                                         // cylinder spanning angle
  shielding_2_log = new G4LogicalVolume(shielding_2,                                  // logical volume
                                            Air_mat,                                  // material
                            shielding_2->GetName());                                  // name of logical volume (same that solid volume)
  new G4PVPlacement(NULL,                                                             // no rotation
                    G4ThreeVector(0,0,0),                                             // position (the top of germanium Cristal is at zero according to symmetry axis)
                    shielding_2_log,                                                  // the logical volume
                    shielding_2->GetName(),                                           // name of physical volume (same that solid volume)
                    shielding_1_log,                                                  // the mother volume
                    false,                                                            // no boolean ops
                    0,                                                                // copy number
                    overlapChecking);                                                 // check for overlapping
  G4Colour lbrown(0.6, 0.6, 0.2);                                                     // visualization attributes
  G4VisAttributes* vaLBrown = new G4VisAttributes(lbrown);
  vaLBrown->SetVisibility(1);
  vaLBrown->SetForceSolid(forceSolid);
  vaLBrown->SetForceAuxEdgeVisible(1);
  G4Colour cyan(0.0, 1.0, 1.0);
  G4VisAttributes* vaCyan = new G4VisAttributes(cyan);
  vaCyan->SetVisibility(1);
  vaCyan->SetForceSolid(forceSolid);
  vaCyan->SetForceAuxEdgeVisible(1);

  shielding_1_log->SetVisAttributes(vaLBrown);
  shielding_2_log->SetVisAttributes(vaCyan);

  } // end shield geo

  // create HPGe detector (equivalent : Detector A and B)
  
  // HPGe window
  
  G4double windowSize = 80.0*mm;
  G4double windowLayer = 1.0*mm;
  
  G4Tubs* ge_window_1 = new G4Tubs("HPGe_WINDOW",                                     // solid volume name
                                   0.0,                                               // cylinder inner radius
                                   windowSize/2,                                      // cylinder outer radius
                                   windowSize/2,                                      // cylinder half Z
                                   startAng,                                          // cylinder start angle
                                   spaceAng);                                         // cylinder spanning angle
  G4LogicalVolume* ge_window_1_log = new G4LogicalVolume(ge_window_1,                 // logical volume
                                                         Aluminum_mat,                // material
                                                         ge_window_1->GetName());     // name of logical volume (same that solid volume)
  if (addShielding==true)
  {
    new G4PVPlacement(NULL,                                                           // no rotation
                    G4ThreeVector(0,0,0),//-windowSize/2),                            // position (the top of germanium Cristal is at zero according to symmetry axis)
                    ge_window_1_log,                                                  // the logical volume
                    ge_window_1->GetName(),                                           // name of physical volume (same that solid volume)
                    shielding_2_log,                                                  // the mother volume
                    false,                                                            // no boolean ops
                    0,                                                                // copy number
                    overlapChecking);                                                 // check for overlapping
  } 
  else
  {
    new G4PVPlacement(NULL,                                                           // no rotation
                    G4ThreeVector(0,0,0),//-windowSize/2),                            // position (the top of germanium Cristal is at zero according to symmetry axis)
                    ge_window_1_log,                                                  // the logical volume
                    ge_window_1->GetName(),                                           // name of physical volume (same that solid volume)
                    logicalWorld,                                                     // the mother volume
                    false,                                                            // no boolean ops
                    0,                                                                // copy number
                    overlapChecking);                                                 // check for overlapping
  }
  G4Tubs* ge_window_2 = new G4Tubs("HPGe_HOUSE",                                      // solid volume name
                                   0.0,                                               // cylinder inner radius
                                   windowSize/2 -windowLayer,                         // cylinder outer radius
                                   windowSize/2 -windowLayer,                         // cylinder half Z
                                   startAng,                                          // cylinder start angle
                                   spaceAng);                                         // cylinder spanning angle
  G4LogicalVolume* ge_window_2_log = new G4LogicalVolume(ge_window_2,                 // logical volume
                                                         Vacuum_mat,                  // material
                                                         ge_window_2->GetName());     // name of logical volume (same that solid volume)
  new G4PVPlacement(NULL,                                                             // no rotation
                    G4ThreeVector(0,0,0),                                             // position
                    ge_window_2_log,                                                  // the logical volume
                    ge_window_2->GetName(),                                           // name of physical volume (same that solid volume)
                    ge_window_1_log,                                                  // the mother volume
                    false,                                                            // no boolean ops
                    0,                                                                // copy number
                    overlapChecking);                                                 // check for overlapping
  G4Colour blue(0.0, 0.0, 1.0);                                                       // visualization attributes
  G4VisAttributes* vaBlue = new G4VisAttributes(blue);
  vaBlue->SetVisibility(1);
  vaBlue->SetForceSolid(forceSolid);
  vaBlue->SetForceAuxEdgeVisible(1);
  G4Colour white(1.0, 1.0, 1.0);
  G4VisAttributes* vaWhite = new G4VisAttributes(white);
  vaWhite->SetVisibility(1);
  vaWhite->SetForceSolid(forceSolid);
  vaWhite->SetForceAuxEdgeVisible(1);
  
  ge_window_1_log->SetVisAttributes(vaBlue);
  ge_window_2_log->SetVisAttributes(vaWhite);
  
  // create Cristal Ge
  
  G4double cristalSize = 60.0*mm;  // default value set to 60.0*mm
  
  if(addGeDeadZone==true)
  {
      G4cout<<"==========================================="<<G4endl;
      G4cout<<" Build HPGe detector"<<"\n"
            <<" Equivalent : Detector A " <<"\n"
            <<" This geometry consists of a bare germanium crystal (60x60 mm, density 5.323 g/cm3)"<<"\n"
            <<" and a point source located 1 mm (default value) above the crystal and on its symmetry axis"<<G4endl;
      G4cout<<" Dead zone thickness set in this run : "<< G4BestUnit(cristalDZLayer,"Length")<<G4endl;
      G4cout<<"==========================================="<<G4endl;
  
      // Dead Zone                                                                    // default value set to 1mm-thick
      G4Tubs* ge_crystalDZ = new G4Tubs("HPGe_CRISTAL_DZ",                            // solid volume name
                                        0.0,                                          // cylinder inner radius
                                        cristalSize/2,                                // cylinder outer radius
                                        cristalSize/2,                                // cylinder half Z
                                        startAng,                                     // cylinder start angle
                                        spaceAng);                                    // cylinder spanning angle
      G4LogicalVolume* ge_crystalDZ_log = new G4LogicalVolume(ge_crystalDZ,           // logical volume
                                                              Germanium_mat,          // material
                                                            ge_crystalDZ->GetName()); // name of logical volume (same to solid volume)
      new G4PVPlacement(NULL,                                                         // no rotation
                      G4ThreeVector(0,0,windowSize/2-windowLayer-cristalSize/2-5*mm), // position
                        ge_crystalDZ_log,                                             // the logical volume
                        ge_crystalDZ->GetName(),                                      // name of physical volume (same to solid volume)
                        ge_window_2_log,                                              // the mother volume
                        false,                                                        // no boolean ops
                        0,                                                            // copy number
                        overlapChecking);                                             // check for overlapping
  
      // ACTIVE = sensitive detector
  
      G4Tubs* ge_crystalSD = new G4Tubs("HPGe_CRISTAL_SD",                            // solid volume name
                                        0.0,                                          // cylinder inner radius
                                        cristalSize/2-cristalDZLayer,                 // cylinder outer radius
                                        cristalSize/2-cristalDZLayer/2,               // cylinder half Z
                                        startAng,                                     // cylinder start angle
                                        spaceAng);                                    // cylinder spanning angle
      G4LogicalVolume* ge_crystalSD_log = new G4LogicalVolume(ge_crystalSD,           // logical volume
                                                              Germanium_mat,          // material
                                                            ge_crystalSD->GetName()); // name of logical volume (same that solid volume)
      new G4PVPlacement(NULL,                                                         // no rotation
                        G4ThreeVector(0,0,-cristalDZLayer/2),                         // position
                        ge_crystalSD_log,                                             // the logical volume
                        ge_crystalSD->GetName(),                                      // name of physical volume (same that solid volume)
                        ge_crystalDZ_log,                                             // the mother volume
                        false,                                                        // no boolean ops
                        0,                                                            // copy number
                        overlapChecking);                                             // check for overlapping
      G4Colour brown(0.7, 0.4, 0.1);                                                  // visualization attributes
      G4VisAttributes* vaBrown = new G4VisAttributes(brown);
      vaBrown->SetVisibility(1);
      vaBrown->SetForceSolid(forceSolid);
      vaBlue->SetForceAuxEdgeVisible(1);
      G4Colour yellow(1.0, 1.0, 0.0);
      G4VisAttributes* vaYellow = new G4VisAttributes(yellow);
      vaYellow->SetVisibility(1);
      vaYellow->SetForceSolid(forceSolid);
      vaYellow->SetForceAuxEdgeVisible(1);
      
      ge_crystalDZ_log->SetVisAttributes(vaBrown);
      ge_crystalSD_log->SetVisAttributes(vaYellow);

      // Cristal Ge Hole

      // size = 10 x 40 mm
      G4double hole_diameter = 10.0*mm;
      G4double hole_height = 40.0*mm;
      G4Tubs* ge_hole = new G4Tubs("HPGe_HOLE",                                       // solid volume name
                                   0.0,                                               // cylinder inner radius
                                   hole_diameter/2,                                   // cylinder outer radius
                                   hole_height/2,                                     // cylinder half Z
                                   startAng,                                          // cylinder start angle
                                   spaceAng);                                         // cylinder spanning angle
      G4LogicalVolume* ge_hole_log = new G4LogicalVolume(ge_hole,                     // logical volume
                                                         Vacuum_mat,                  // material
                                                         ge_hole->GetName());         // name of logical volume (same that solid volume)
      new G4PVPlacement(NULL,                                                         // no rotation
                    G4ThreeVector(0,0,-cristalSize/2+cristalDZLayer/2+hole_height/2), // position
                        ge_hole_log,                                                  // the logical volume
                        ge_hole->GetName(),                                           // name of physical volume (same that solid volume)
                        ge_crystalSD_log,                                             // the mother volume
                        false,                                                        // no boolean ops
                        0,                                                            // copy number
                        overlapChecking);                                             // check for overlapping
      G4Colour cyan1(0.0, 1.0, 1.0);                                                  // visualization attributes
      G4VisAttributes* vaGrey = new G4VisAttributes(cyan1);
      vaGrey->SetVisibility(1);
      vaGrey->SetForceSolid(forceSolid);
      vaGrey->SetForceAuxEdgeVisible(1);
      
      ge_hole_log->SetVisAttributes(vaGrey);

  }
  else 
  {

      G4cout<<"==========================================="<<G4endl;
      G4cout<<" Build HPGe detector "<<"\n"
           <<" Equivalent : Detector B " <<"\n"
          <<" This geometry consists of a bare germanium crystal (60x60 mm, density 5.323 g/cm3)"<<"\n"
         <<" without a dead zone "<<G4endl;

      // ACTIVE = sensitive detector

      G4Tubs* ge_crystalSD = new G4Tubs("HPGe_CRISTAL_SD", 
                                        0.0,               
                                        cristalSize/2,     
                                        cristalSize/2,     
                                        startAng,          
                                        spaceAng);         

      G4LogicalVolume* ge_crystalSD_log = new G4LogicalVolume(ge_crystalSD, 
                                                              Germanium_mat,                  
                                                              ge_crystalSD->GetName());
      new G4PVPlacement(NULL,                                             
                        G4ThreeVector(0,0,windowSize/2 -windowLayer -cristalSize/2 -5.0*mm),  
                        ge_crystalSD_log,                                                     
                        ge_crystalSD->GetName(),                                              
                        ge_window_2_log,                                                      
                        false,                                                                
                        0,                                                                    
                        overlapChecking);                                                     
      G4Colour yellow(1.0, 1.0, 0.0);                                       
      G4VisAttributes* vaYellow = new G4VisAttributes(yellow);
      vaYellow->SetVisibility(1);
      vaYellow->SetForceSolid(forceSolid);
      vaYellow->SetForceAuxEdgeVisible(1);
    
      ge_crystalSD_log->SetVisAttributes(vaYellow);

      // Cristal Ge Hole

      // size = 10 x 40 mm
      G4double hole_diameter = 10.0*mm;
      G4double hole_height = 40.0*mm;
      G4Tubs* ge_hole = new G4Tubs("HPGe_HOLE",       
                                   0.0,               
                                   hole_diameter/2,   
                                   hole_height/2,     
                                   startAng,          
                                   spaceAng);         
      G4LogicalVolume* ge_hole_log = new G4LogicalVolume(ge_hole,            
                                                         Vacuum_mat,         
                                                         ge_hole->GetName());
      new G4PVPlacement(NULL,                                                
                        G4ThreeVector(0,0,-cristalSize/2 +hole_height/2),    
                        ge_hole_log,                                         
                        ge_hole->GetName(),                                  
                        ge_crystalSD_log,                                    
                        false,                                               
                        0,                                                   
                        overlapChecking);                                    
      G4Colour cyan2(0.0, 1.0, 1.0);                                         
      G4VisAttributes* vaGrey = new G4VisAttributes(cyan2);
      vaGrey->SetVisibility(1);
      vaGrey->SetForceSolid(forceSolid);
      vaGrey->SetForceAuxEdgeVisible(1);
      
      ge_hole_log->SetVisAttributes(vaGrey);
  }

  // Add source volume
  
  // Set right source position 
  sourceTranslation = sourcePosition + G4ThreeVector(0,0,windowSize/2); // translation according to HPGe size and user defined position 

  if(sourceType=="Point")
  {
      G4cout<<""<<G4endl;
      G4cout<<" Source type : a point source located at 1 mm (default value) from the detector window "<<G4endl;
  }
  else if(sourceType=="Water")
  {
      G4cout<<""<<G4endl;
      G4cout<<" Source type : a point source located at 1 mm (default value) from the detector window "<<G4endl;

      // For Water source volume

      source_diameter=90.0*mm;
      source_height=40.0*mm;

      G4Tubs* source_water = new G4Tubs("SOURCE_VOLUME", 0, source_diameter/2, source_height/2, startAng, spaceAng);        

      G4LogicalVolume* source_water_log = new G4LogicalVolume(source_water, Water_mat, source_water->GetName());    

      if (addShielding==true)
      { 
      	new G4PVPlacement(NULL, G4ThreeVector(0,0,source_height/2)+sourceTranslation, source_water_log, source_water->GetName(), shielding_2_log, false, 0, overlapChecking);                                     
      }
      else
      {
      	new G4PVPlacement(NULL, G4ThreeVector(0,0,source_height/2)+sourceTranslation, source_water_log, source_water->GetName(), logicalWorld, false, 0, overlapChecking);                                     
      }
      G4Colour red(1.0, 0.3, .3);                                             
      G4VisAttributes* vaLBlue = new G4VisAttributes(red);
      vaLBlue->SetVisibility(1);
      vaLBlue->SetForceSolid(forceSolid);
      vaLBlue->SetForceAuxEdgeVisible(1);
      
      source_water_log->SetVisAttributes(vaLBlue);
  }
  else if(sourceType=="Soil")
  {
      G4cout<<""<<G4endl;
      G4cout<<" Source type : a point source located at 1 mm  (default value) from the detector window "<<G4endl;

      // For Soil source volume

      source_diameter=60.0*mm;
      source_height=20.0*mm;

      G4Tubs* source_water = new G4Tubs("SOURCE_VOLUME", 0, source_diameter/2, source_height/2, startAng, spaceAng);        

      G4LogicalVolume* source_water_log = new G4LogicalVolume(source_water, Dirt_mat, source_water->GetName());

      if (addShielding==true)
      { 
      	new G4PVPlacement(NULL, G4ThreeVector(0,0,source_height/2)+sourceTranslation, source_water_log, source_water->GetName(), shielding_2_log, false, 0, overlapChecking);                                              
      }
      else 
      {
      	new G4PVPlacement(NULL, G4ThreeVector(0,0,source_height/2)+sourceTranslation, source_water_log, source_water->GetName(), logicalWorld, false, 0, overlapChecking);                                              
      }
      G4Colour magenta(1.0, 0.0, 1.0);                                
      G4VisAttributes* vaLBlue = new G4VisAttributes(magenta);
      vaLBlue->SetVisibility(1);
      vaLBlue->SetForceSolid(forceSolid);
      vaLBlue->SetForceAuxEdgeVisible(1);
      
      source_water_log->SetVisAttributes(vaLBlue);
  }
  else if(sourceType=="Filter")
  {

      G4cout<<""<<G4endl;
      G4cout<<" Source type : a point source located at 1 mm (default value) from the detector window "<<G4endl;

      // For Filter source volume

      source_diameter=80.0*mm;
      source_height=3.0*mm;

      G4Tubs* source_water = new G4Tubs("SOURCE_VOLUME", 0.0, source_diameter/2, source_height/2, startAng, spaceAng);        

      G4LogicalVolume* source_water_log = new G4LogicalVolume(source_water, Cellulose_mat, source_water->GetName());
      if (addShielding==true)
      {
      	new G4PVPlacement(NULL, G4ThreeVector(0,0,source_height/2)+sourceTranslation, source_water_log, source_water->GetName(), shielding_2_log, false, 0, overlapChecking);                                              
      }
      else
      {
		new G4PVPlacement(NULL, G4ThreeVector(0,0,source_height/2)+sourceTranslation, source_water_log, source_water->GetName(), logicalWorld, false, 0, overlapChecking);                                              
      }
      G4Colour purple(0.5, 0.0, 1.5);                                   
      G4VisAttributes* vaLBlue = new G4VisAttributes(purple);
      vaLBlue->SetVisibility(1);
      vaLBlue->SetForceSolid(forceSolid);
      vaLBlue->SetForceAuxEdgeVisible(1);
      
      source_water_log->SetVisAttributes(vaLBlue);
  }
  else G4Exception("GRSDetectorConstruction::ConstructDetector()", "Source type not found", EventMustBeAborted, "This run will be aborted");

  // print all setting parameters
 // if(verbose==1){
      G4cout << " ---Gamma-Ray-Spectrometry--- RunStatus "<<G4endl;
      G4cout << "    verbose                      : "<<std::setw(12)<< verbose<<G4endl;
      G4cout << "    cylinders start angle        : "<<std::setw(12)<< G4BestUnit(startAng,"Angle")<<G4endl;
      G4cout << "    cylinders spacing angle      : "<<std::setw(12)<< G4BestUnit(spaceAng,"Angle")<<G4endl;
      G4cout << "    geometry force solid         : "<<std::setw(12)<< forceSolid <<G4endl;
      G4cout << "    Lead shielding               : "<<std::setw(12)<< addShielding <<G4endl;
      G4cout << "    ge-Cristal dead zone         : "<<std::setw(12)<< addGeDeadZone <<G4endl;
      G4cout << "    ge-Cristal dead zone thick   : "<<std::setw(12)<< G4BestUnit(cristalDZLayer,"Length") <<G4endl;
      G4cout << "    source type                  : "<<std::setw(12)<< sourceType <<G4endl;
      G4cout << "    source diameter              : "<<std::setw(12)<< G4BestUnit(source_diameter,"Length") <<G4endl;
      G4cout << "    source height                : "<<std::setw(12)<< G4BestUnit(source_height,"Length") <<G4endl;
      G4cout << "    prim pos.( sourceTranslation): "<<std::setw(12)<< G4BestUnit(sourceTranslation,"Length")<<G4endl;
      G4cout << "    primary particle name        : "<<std::setw(12)<< particleName <<G4endl;
      G4cout << "    primary particle energy      : "<<std::setw(12)<< G4BestUnit(particleEnergy,"Energy")<<G4endl;
      G4cout << "    peak energy sigma            : "<<std::setw(12)<< G4BestUnit(energySigma,"Energy")<<G4endl;
      G4cout << "    detection threshold          : "<<std::setw(12)<< G4BestUnit(detecEThreshold,"Energy")<<G4endl;
      G4cout << "    add output                   : "<<std::setw(12)<< addOutputFile<<G4endl;
      G4cout << "    output file name             : "<<std::setw(12)<< fileName<<G4endl;
      G4cout << " "<<G4endl;
   //}

   // end of detector construction
  return physicalWorld; // always return this
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSDetectorConstruction::SetVerboseLevel(G4int value)
{ 
 verbose=value; updated=true; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetGeoForceSolid(G4bool value)
{ 
 forceSolid=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetGeoSpaceAngle(G4double value)
{ 
  spaceAng=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::AddOutputFile(G4bool value)
{ 
addOutputFile=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetFileName(G4String value)
{ 
fileName=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::AddShielding(G4bool value)
{ 
addShielding=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetNameOfParticle(G4String value)
{
 particleName=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetAOfIon(G4int value) 
{ 
 ionA = value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetZOfIon(G4int value) 
{ 
 ionZ = value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetEnergyOfParticle(G4double value)
{ 
 particleEnergy=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetPeakEnergySigma(G4double value)
{ 
 energySigma=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetDetectionEnergyThreshold(G4double value)
{ 
 detecEThreshold=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetSourcePosition(G4ThreeVector value)
{ 
 sourcePosition=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetSourceType(G4String value)
{ 
 sourceType=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::AddGermaniumDeadZone(G4bool value)
{ 
 addGeDeadZone=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

void GRSDetectorConstruction::SetGeoIGermaniumDZLayer(G4double value)
{ 
 cristalDZLayer=value; G4RunManager::GetRunManager()->ReinitializeGeometry();
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//
