//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// \file GammaRaySpectrometry/src/GRSDetectorMessenger.cc
// \brief Implementation of the DetectorMessenger class

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

#include "GRSDetectorMessenger.hh"

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSDetectorMessenger::GRSDetectorMessenger(GRSDetectorConstruction* GRSDetect):GRSDetector(GRSDetect)
{
  // Private command directory for GRS construction
  GRSDir = new G4UIdirectory("/GRS/");
  GRSDir->SetGuidance("GRS control commands.");

  // Setup a command directory for detector controls with guidance
  detectorDir = new G4UIdirectory("/GRS/detector/");
  detectorDir->SetGuidance("GRS detector control commands.");

  // verbose
  verboseCmd = new G4UIcmdWithAnInteger("/GRS/detector/Verbose",this);
  verboseCmd->SetGuidance("GRS detector construction verbose");
  verboseCmd->SetGuidance(" - verbose=0 check overlapping in geometry construction");
  verboseCmd->SetGuidance(" - verbose=0 print list of materials");
  verboseCmd->SetGuidance(" - verbose=1 print all setting parameters");
  verboseCmd->SetGuidance(" - verbose=2 print counting information by event % print modulo value");
  verboseCmd->SetParameterName("verb",true);
  verboseCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  verboseCmd->SetToBeBroadcasted(true);

  // germanium Cristal dead zone layer
  dzLayerCmd = new G4UIcmdWithADoubleAndUnit("/GRS/detector/GeCristalDeadZone",this);
  dzLayerCmd->SetGuidance("Set dead zone thick for germanium crystal");
  dzLayerCmd->SetGuidance("(default : diameter = height = 60 mm");
  dzLayerCmd->SetParameterName("dzSize",false);
  dzLayerCmd->SetUnitCategory("Length");
  dzLayerCmd->SetRange("dzSize>= 0.0 && dzSize <= 10.0");
  dzLayerCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  dzLayerCmd->SetToBeBroadcasted(true);
  
  // Add Ge dead zone
  adzCmd = new G4UIcmdWithABool("/GRS/detector/AddGermaniumDeadZone",this);
  adzCmd->SetGuidance("Add germanium dead zone in geometry construction");
  adzCmd->SetGuidance("default : true with 1 mm thick");
  adzCmd->SetParameterName("ADZ",true);
  adzCmd->SetDefaultValue(true);
  adzCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  adzCmd->SetToBeBroadcasted(true);

   // Add detector shielding
  adsCmd = new G4UIcmdWithABool("/GRS/detector/AddShielding",this);
  adsCmd->SetGuidance("Add lead shielding in geometry construction");
  adsCmd->SetGuidance("default : true");
  adsCmd->SetParameterName("ADS",true);
  adsCmd->SetDefaultValue(true);
  adsCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  adsCmd->SetToBeBroadcasted(false);

  // geometry force solid
  gfsCmd = new G4UIcmdWithABool("/GRS/detector/GeometryForceSolid",this);
  gfsCmd->SetGuidance("Set geometry force solid true or false");
  gfsCmd->SetGuidance("default : false");
  gfsCmd->SetParameterName("GSF",true);
  gfsCmd->SetDefaultValue(true);
  gfsCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  gfsCmd->SetToBeBroadcasted(true);

  // geometry spacing angle
  spaceAngCmd = new G4UIcmdWithADoubleAndUnit("/GRS/detector/GeometrySpaceAngle",this);
  spaceAngCmd->SetGuidance("Set spacing angle for cylinders");
  spaceAngCmd->SetGuidance("can be changed only for visualization - default : 360*deg");
  spaceAngCmd->SetParameterName("angle",true,true);
  spaceAngCmd->SetDefaultUnit("deg");
  spaceAngCmd->SetUnitCategory("Angle");
  spaceAngCmd->SetUnitCandidates("deg");
  spaceAngCmd->SetRange("angle > 0. && angle <= 360.0" );
  spaceAngCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  spaceAngCmd->SetToBeBroadcasted(true);

  // source position
  srcPosCmd = new G4UIcmdWith3VectorAndUnit("/GRS/detector/SourceParticlePosition",this);
  srcPosCmd->SetGuidance("Set source particle position - threeVector x y z");
  srcPosCmd->SetGuidance("Source position is according to the Z axis");
  srcPosCmd->SetParameterName("X","Y","Z",true,true);
  srcPosCmd->SetDefaultUnit("mm");
  srcPosCmd->SetUnitCandidates("micron mm cm m km");
  srcPosCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  srcPosCmd->SetToBeBroadcasted(false);

  // primary particle name
  srcTypeCmd = new G4UIcmdWithAString("/GRS/detector/SourceType",this);
  srcTypeCmd->SetGuidance("Set source type");
  srcTypeCmd->SetGuidance("default : Point");
  srcTypeCmd->SetGuidance("if Point : point source positioned at 0 0 z=1mm default");
  srcTypeCmd->SetGuidance("Water : volume source 90x40 mm is constructed and particles are randomly generated inside");
  srcTypeCmd->SetGuidance("Soil : volume source 60x20 mm is constructed and particles are randomly generated inside");
  srcTypeCmd->SetGuidance("Filter : volume source 80x3 mm is constructed and particles are randomly generated inside");
  srcTypeCmd->SetParameterName("srcType",true,true);
  srcTypeCmd->SetDefaultValue("Point");
  srcTypeCmd->SetCandidates("Point Water Soil Filter");
  srcTypeCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  srcTypeCmd->SetToBeBroadcasted(false);

  // primary particle name
  particleCmd = new G4UIcmdWithAString("/GRS/detector/SourceParticleName",this);
  particleCmd->SetGuidance("Set source particle type");
  particleCmd->SetParameterName("pName",true,true);
  particleCmd->SetDefaultValue("geantino");
  particleCmd->SetCandidates("geantino e- e+ gamma ion nuclide");
  particleCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  particleCmd->SetToBeBroadcasted(false);

  // primary ion atomic mass
  ionatomicmassCmd = new G4UIcmdWithAnInteger("/GRS/detector/IonAtomicMass", this);
  ionatomicmassCmd->SetGuidance("Set ion atomic mass");
  ionatomicmassCmd->SetParameterName("A", true);
  ionatomicmassCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  ionatomicmassCmd->SetToBeBroadcasted(false);

  // primary ion atomic number
  ionatomicnumberCmd = new G4UIcmdWithAnInteger("/GRS/detector/IonAtomicNumber", this);
  ionatomicnumberCmd->SetGuidance("Set ion atomic number");
  ionatomicnumberCmd->SetParameterName("Z", true);
  ionatomicnumberCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  ionatomicnumberCmd->SetToBeBroadcasted(false);

  // primary particle energy
  energyCmd = new G4UIcmdWithADoubleAndUnit("/GRS/detector/SourceParticleEnergy",this);
  energyCmd->SetGuidance("Set kinetic energy for source particle");
  energyCmd->SetParameterName("Energy",true,true);
  energyCmd->SetDefaultUnit("keV");
  energyCmd->SetUnitCategory("Energy");
  energyCmd->SetUnitCandidates("eV keV MeV GeV TeV");
  energyCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  energyCmd->SetToBeBroadcasted(false);

  // peak energy sigma
  eSigmaCmd = new G4UIcmdWithADoubleAndUnit("/GRS/detector/PeakEnergySigma",this);
  eSigmaCmd->SetGuidance("Set sigma value for peak-energy efficiency calculation");
  eSigmaCmd->SetParameterName("ESIG",true,true);
  eSigmaCmd->SetDefaultUnit("keV");
  eSigmaCmd->SetUnitCategory("Energy");
  eSigmaCmd->SetUnitCandidates("eV keV MeV GeV TeV");
  eSigmaCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  eSigmaCmd->SetToBeBroadcasted(false);

  // detection energy threshold
  eThrCmd = new G4UIcmdWithADoubleAndUnit("/GRS/detector/DetectionEnergyThreshold",this);
  eThrCmd->SetGuidance("Set energy detection threshold for efficiency calculation");
  eThrCmd->SetParameterName("ETH",true,true);
  eThrCmd->SetDefaultUnit("keV");
  eThrCmd->SetUnitCategory("Energy");
  eThrCmd->SetUnitCandidates("eV keV MeV GeV TeV");
  eThrCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  eThrCmd->SetToBeBroadcasted(false);

  // output
  outputCmd = new G4UIcmdWithABool("/GRS/detector/AddOutputFile",this);
  outputCmd->SetGuidance("Add additional ASCII output file");
  outputCmd->SetParameterName("OUT",true);
  outputCmd->SetDefaultValue(true);
  outputCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  outputCmd->SetToBeBroadcasted(false);

  outputNameCmd = new G4UIcmdWithAString("/GRS/detector/OutputFileName",this);
  outputNameCmd->SetGuidance("Set output file name ");
  outputNameCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  outputNameCmd->SetToBeBroadcasted(false);
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSDetectorMessenger::~GRSDetectorMessenger()
{
 delete GRSDir;
 delete detectorDir;
 delete verboseCmd;
 delete outputNameCmd;
 delete outputCmd;
 delete ionatomicmassCmd;
 delete ionatomicnumberCmd;
 delete particleCmd;
 delete energyCmd;
 delete eSigmaCmd;
 delete eThrCmd;
 delete srcPosCmd;
 delete dzLayerCmd;
 delete adzCmd;
 delete adsCmd;
 delete srcTypeCmd;
 delete gfsCmd;
 delete spaceAngCmd;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSDetectorMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
 if(command == verboseCmd) { GRSDetector->SetVerboseLevel(verboseCmd->GetNewIntValue(newValue));}

 if(command == adzCmd) { GRSDetector->AddGermaniumDeadZone(adzCmd->GetNewBoolValue(newValue));}

 if(command == adsCmd) { GRSDetector->AddShielding(adsCmd->GetNewBoolValue(newValue));}

 if(command == gfsCmd) { GRSDetector->SetGeoForceSolid(gfsCmd->GetNewBoolValue(newValue));}

 if(command == spaceAngCmd) { GRSDetector->SetGeoSpaceAngle(spaceAngCmd->GetNewDoubleValue(newValue));}

 if(command == dzLayerCmd) { GRSDetector->SetGeoIGermaniumDZLayer(dzLayerCmd->GetNewDoubleValue(newValue));}

 if(command == srcPosCmd) { GRSDetector->SetSourcePosition(srcPosCmd->GetNew3VectorValue(newValue));}

 if(command == energyCmd) { GRSDetector->SetEnergyOfParticle(energyCmd->GetNewDoubleValue(newValue));}

 if(command == eSigmaCmd) { GRSDetector->SetPeakEnergySigma(eSigmaCmd->GetNewDoubleValue(newValue));}

 if(command == eThrCmd) { GRSDetector->SetDetectionEnergyThreshold(eThrCmd->GetNewDoubleValue(newValue));}

 if(command == particleCmd) { GRSDetector->SetNameOfParticle(newValue);}

 if (command == ionatomicmassCmd) { GRSDetector->SetAOfIon(ionatomicmassCmd->GetNewIntValue(newValue));}

 if (command == ionatomicnumberCmd) { GRSDetector->SetZOfIon(ionatomicnumberCmd->GetNewIntValue(newValue));}

 if(command ==srcTypeCmd) { GRSDetector->SetSourceType(newValue);}

 if(command == outputCmd) { GRSDetector->AddOutputFile(outputCmd->GetNewBoolValue(newValue));}

 if(command == outputNameCmd) { GRSDetector->SetFileName(newValue);}
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//
