//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// \file GammaRaySpectrometry/src/GRSPhysicsList.cc
// \brief Implementation of the PhysicsList class

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

#include "GRSPhysicsList.hh"

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSPhysicsList::GRSPhysicsList():G4VModularPhysicsList()
{
  pMessenger = new GRSPhysicsListMessenger(this);
  
  // EM physics
  emName = G4String("emlivermore");
  emPhysicsList = new G4EmLivermorePhysics();
 
  // default cut value  
  defaultCutValue = 0.01*um;

  cutForGamma     = defaultCutValue;
  cutForElectron  = defaultCutValue;
  cutForPositron  = defaultCutValue;
  
  SetVerboseLevel(1); 

  G4ProductionCutsTable::GetProductionCutsTable()->SetEnergyRange(100*eV, 100*TeV);
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSPhysicsList::~GRSPhysicsList()
{
  delete pMessenger;
  delete emPhysicsList;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPhysicsList::ConstructParticle()
{
  // pseudo-particles
  G4Geantino::GeantinoDefinition();
  G4ChargedGeantino::ChargedGeantinoDefinition();
  
  // gamma
  G4Gamma::GammaDefinition();
  
  // leptons
  G4Electron::ElectronDefinition();
  G4Positron::PositronDefinition();
  G4MuonPlus::MuonPlusDefinition();
  G4MuonMinus::MuonMinusDefinition();

  G4NeutrinoE::NeutrinoEDefinition();
  G4AntiNeutrinoE::AntiNeutrinoEDefinition();
  G4NeutrinoMu::NeutrinoMuDefinition();
  G4AntiNeutrinoMu::AntiNeutrinoMuDefinition();  

  // mesons
  G4MesonConstructor mConstructor;
  mConstructor.ConstructParticle();

  // barions
  G4BaryonConstructor bConstructor;
  bConstructor.ConstructParticle();

  // ions
  G4IonConstructor iConstructor;
  iConstructor.ConstructParticle();
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPhysicsList::ConstructProcess()
{
    G4cout<< " " << G4endl;
    G4cout<< " PhysicsList construction... " << G4endl;
    
    AddTransportation();
    
    ConstructGeneral();

    emPhysicsList->ConstructProcess();
 
   // EM options

   G4EmParameters* emOptions = G4EmParameters::Instance();

   emOptions->SetBuildCSDARange(true);
   emOptions->SetFluo(1);
   emOptions->SetAuger(1);
   emOptions->SetPixe(0);
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPhysicsList::ConstructGeneral() 
{
  // add Decay Process
  G4Decay* theDecayProcess = new G4Decay();
  auto theParticleIterator = GetParticleIterator();
  theParticleIterator->reset();
  while( (*theParticleIterator)() )
  {
      G4ParticleDefinition* particle = theParticleIterator->value();
      G4ProcessManager* pmanager = particle->GetProcessManager();
      if (theDecayProcess->IsApplicable(*particle) && !particle->IsShortLived()) 
      { 
       pmanager ->AddProcess(theDecayProcess);
       // set ordering for PostStepDoIt and AtRestDoIt
       pmanager ->SetProcessOrdering(theDecayProcess, idxPostStep);
       pmanager ->SetProcessOrdering(theDecayProcess, idxAtRest);
      }
  }
    
  // declare radioactive decay to the GenericIon in the IonTable.
  const G4IonTable *theIonTable = G4ParticleTable::GetParticleTable()->GetIonTable();
  G4RadioactiveDecay *theRadioactiveDecay = new G4RadioactiveDecay();

  for (G4int i=0; i<theIonTable->Entries(); i++) 
  {
      G4String particleName = theIonTable->GetParticle(i)->GetParticleName();
      G4String particleType = theIonTable->GetParticle(i)->GetParticleType();
      
      if (particleName == "GenericIon") 
	    {
	      G4ProcessManager* pmanager = theIonTable->GetParticle(i)->GetProcessManager();
	      pmanager->SetVerboseLevel(1);
	      pmanager ->AddProcess(theRadioactiveDecay);
	      pmanager ->SetProcessOrdering(theRadioactiveDecay, idxPostStep);
	      pmanager ->SetProcessOrdering(theRadioactiveDecay, idxAtRest);
	    } 
   }
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPhysicsList::AddPhysicsList(const G4String& name)
{
  if (verboseLevel>-1) G4cout <<" ==> GRSPhysicsList::AddPhysicsList() < " << name << " > " << G4endl;
  
  if (name==emName) return;

  if ((name=="emstandard")||(name=="standard"))
  {
    emName = name;
    delete emPhysicsList;
    emPhysicsList = new G4EmStandardPhysics();
  } 
  else if ((name=="empenelope")||(name=="penelope"))
  {
    emName = name;
    delete emPhysicsList;
    emPhysicsList = new G4EmPenelopePhysics();
  } 
  else if ((name=="emlivermore")||(name=="livermore"))
  {	
    emName = name;
    delete emPhysicsList;
    emPhysicsList = new G4EmLivermorePhysics();                  
  } 
  else if ((name=="emlowenergy")||(name=="lowenergy"))
  { 
    emName = name;
    delete emPhysicsList;
    emPhysicsList = new G4EmLowEPPhysics();                    
  } 
  else 
  {
    G4cout << "PhysicsList::AddPhysicsList() <" << name << ">"<< " is not defined"<< G4endl;
  }
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//


void GRSPhysicsList::SetCutForGamma(G4double cut)
{
  cutForGamma = cut;
  SetParticleCuts(cutForGamma, G4Gamma::Gamma());
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPhysicsList::SetCutForElectron(G4double cut)
{
  cutForElectron = cut;
  SetParticleCuts(cutForElectron, G4Electron::Electron());
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPhysicsList::SetCutForPositron(G4double cut)
{
  cutForPositron = cut;
  SetParticleCuts(cutForPositron, G4Positron::Positron());
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPhysicsList::SetCuts()
{
  if (verboseLevel >0)
  {
   G4cout << "GRSPhysicsList::SetCuts:";
   G4cout << "CutLength : " << G4BestUnit(defaultCutValue,"Length") << G4endl;
  }
  // set cut values for gamma at first and for e- second and next for e+,
  // because some processes for e+/e- need cut values for gamma
  SetCutValue(cutForGamma, "gamma");
  SetCutValue(cutForElectron, "e-");
  SetCutValue(cutForPositron, "e+");

  if (verboseLevel>0) DumpCutValuesTable();
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//
