//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file GammaRaySpectrometry/src/GRSPrimaryGeneratorAction.cc
/// \brief Implementation of the PrimaryGeneratorAction class

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

#include "GRSPrimaryGeneratorAction.hh"

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSPrimaryGeneratorAction::GRSPrimaryGeneratorAction(GRSDetectorConstruction* det):detector(det)
{
 fSourceSolid=NULL;
 fParticleGun  = new G4ParticleGun(1);
 fParticleGun->SetParticleEnergy(0.0);
 particle_direction = G4ThreeVector(0,0,-1);
 particleTable = G4ParticleTable::GetParticleTable();
 verbose=0;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSPrimaryGeneratorAction::~GRSPrimaryGeneratorAction()
{
 delete fParticleGun;
}
//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
 // set particle direction (isotropic)
 if(detector->GetNameOfParticle()=="geantino")
 {
   fParticleGun->SetParticleMomentumDirection(particle_direction); // direction -Z
 }
 else
 {
   particle_direction.setRThetaPhi(1.0, std::acos(G4UniformRand()*2 -1), (G4UniformRand()*2 -1)*180*deg); // isotropic emission
   fParticleGun->SetParticleMomentumDirection(particle_direction); // direction
 }

 // set source type (point, water, soil or filter)
 if(detector->GetSourceType()=="Point")
 {
  fParticleGun->SetParticlePosition( detector->GetSourceTranslation() ); // position : default value set to 0 0 1 mm
 }
 else if(detector->GetSourceType()=="Water")
 {
  fParticleGun->SetParticlePosition( GetSourceCenter() ); // position related to Water volume-source
 }
 else if(detector->GetSourceType()=="Soil")
 {
  fParticleGun->SetParticlePosition( GetSourceCenter() ); // position  related to Soil volume-source
 }
 else if(detector->GetSourceType()=="Filter")
 {
  fParticleGun->SetParticlePosition( GetSourceCenter() ); // position  related to Filter volume-source
 }
 else G4Exception("GRSPrimaryGeneratorAction::GeneratePrimaries(G4Event*)"," SOURCE type not found", EventMustBeAborted,"This run will be aborted");

 // set particle name
 if (detector->GetNameOfParticle()=="ion")
 {
 	G4ParticleDefinition* ion;
   	ion = G4IonTable::GetIonTable()->GetIon(detector->GetZOfIon(), detector->GetAOfIon(), 0);
   	if (ion == 0)
   	{
     		G4cout << "ion with A=" << detector->GetAOfIon()
     	    	       << " and Z="<< detector->GetZOfIon()<< " does not exist" << G4endl;
     		G4Exception("GRSPrimaryGeneratorAction::GeneratePrimaries(G4Event*)", " ION  not found", EventMustBeAborted, "This run will be aborted");
   	}
   	else 
   	{ 
		fParticleGun->SetParticleDefinition(ion); 
   	}	 
 }
 else
 {
   fParticleGun->SetParticleDefinition(particleTable->FindParticle(detector->GetNameOfParticle())); // particle type
   fParticleGun->SetParticleEnergy(detector->GetEnergyOfParticle()); // particle energy
 }

 // gun primary particle in event
 fParticleGun->GeneratePrimaryVertex(anEvent); // vertex event
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

G4ThreeVector GRSPrimaryGeneratorAction::GetSourceCenter(void)
{
    if(fSourceSolid == NULL)
    {
    	G4LogicalVolume* temp = G4LogicalVolumeStore::GetInstance()->GetVolume("SOURCE_VOLUME");
    	if(temp != NULL)
	{
    		fSourceSolid = dynamic_cast< G4Tubs* >(temp->GetSolid());
    	}
    	if(fSourceSolid == NULL)
    	{
    	G4Exception("GRSPrimaryGeneratorAction::GeneratePrimaries(G4Event*)"," SOURCE volume not found", EventMustBeAborted,"This run will be aborted");
    	}
    }

    // build cylinder volume source  x y z + initial position of source volume in detector construction

    G4ThreeVector RandPos;
    G4double x=0., y=0., z=0.;

    G4double Radius = detector->GetVolumeSourceRadius();
    G4double Halfz =  detector->GetVolumeSourceHalfz();

    x = Radius*2.;
    y = Radius*2.;
    while(((x*x)+(y*y)) > (Radius*Radius))
    {
      x = G4UniformRand();
      y = G4UniformRand();
      z = G4UniformRand();

      x = (x*2.*Radius) - Radius;
      y = (y*2.*Radius) - Radius;
      z = (z*2.*Halfz) - Halfz;
    }
    RandPos.setX(x);
    RandPos.setY(y);
    RandPos.setZ(z);

    // return initial position of source
    return G4PhysicalVolumeStore::GetInstance()->GetVolume("SOURCE_VOLUME")->GetTranslation() + RandPos;
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//
