//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file GammaRaySpectrometry/src/GRSRunAction.cc
/// \brief Implementation of the GRSRunAction class

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

#include "GRSRunAction.hh"

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSRunAction::GRSRunAction(GRSAnalysisManager* histo, GRSDetectorConstruction* det)
:G4UserRunAction(), fAnalysisManager(histo), fDetector(det), fEdep(0.0), peakEfficiency(0), totalEfficiency(0)
{
 // register accumulable to the accumulable manager
 G4AccumulableManager* accumulableManager = G4AccumulableManager::Instance();
 accumulableManager->RegisterAccumulable(fEdep);
 accumulableManager->RegisterAccumulable(peakEfficiency);
 accumulableManager->RegisterAccumulable(totalEfficiency);
 
 // set printing event number per each printModulo events
 G4RunManager::GetRunManager()->SetPrintProgress(100);
 	
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

GRSRunAction::~GRSRunAction()
{}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSRunAction::BeginOfRunAction(const G4Run*)
{ 
    //G4cout << "### Run " << aRun->GetRunID() << " start." << G4endl;

    // inform the runManager to save random number seed
    G4RunManager::GetRunManager()->SetRandomNumberStore(false);
    
    // show Rndm status
    if (IsMaster()) G4Random::showEngineStatus();

    // initialize cumulative quantities
    fEdep=0.0;
    peakEfficiency=0;
    totalEfficiency=0;

    // reset accumulables to their initial values
    G4AccumulableManager* accumulableManager = G4AccumulableManager::Instance();
    accumulableManager->Reset();

    //histograms
    fAnalysisManager->Book();
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSRunAction::FillPerEvent(G4double edep)
{
    // accumulate statistic
    fEdep += edep;
    G4double minE = fDetector->GetEnergyOfParticle() - fDetector->GetPeakEnergySigma();
    G4double maxE = fDetector->GetEnergyOfParticle() + fDetector->GetPeakEnergySigma();

    if((edep > minE)&&(edep < maxE)) peakEfficiency +=1; //CountPeakEfficiency();
    if(edep > fDetector->GetDetectionEnergyThreshold()) totalEfficiency +=1; //CountTotalEfficiency();
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//

void GRSRunAction::EndOfRunAction(const G4Run* aRun)
{
    G4int NbOfEvents= aRun->GetNumberOfEvent();
    if (NbOfEvents==0) return;

    // merge accumulable quantities

    G4AccumulableManager* accumulableManager = G4AccumulableManager::Instance();
    accumulableManager->Merge();

    if (IsMaster())
    {
        // create output files
        if(fDetector->GetAddOutputFile()==true) fAnalysisManager->CreateASCIIDataFile(aRun, totalEfficiency.GetValue(), peakEfficiency.GetValue());
        G4cout
                << G4endl
                << "--------------------End of Global Run-----------------------";
    
        fAnalysisManager->PrintStatistic(aRun, totalEfficiency.GetValue(), peakEfficiency.GetValue());
    }
    else
    {
    	G4cout
     	<< G4endl
     	<< "--------------------End of Local Run------------------------";
    }

 fAnalysisManager->Save();
}

//---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---Gamma-Ray-Spectrometry---%%%---//
